/*! babel-preset-edge v0.7.3 by Sebastian Software <s.werner@sebastian-software.de> */
'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var _typeof = _interopDefault(require('babel-runtime/helpers/typeof'));
var appRootDir = require('app-root-dir');
var path = require('path');
var browserslist = _interopDefault(require('browserslist'));
var envPreset = _interopDefault(require('babel-preset-env'));
var flowPreset = _interopDefault(require('babel-preset-flow'));
var dynamicImportPlugin = _interopDefault(require('babel-plugin-syntax-dynamic-import'));
var dynamicImportNode = _interopDefault(require('babel-plugin-dynamic-import-node'));
var dynamicImportWebpack = _interopDefault(require('babel-plugin-dynamic-import-webpack'));
var moduleResolver = _interopDefault(require('babel-plugin-module-resolver'));
var fastAsyncPlugin = _interopDefault(require('babel-plugin-fast-async'));
var classPropertiesPlugin = _interopDefault(require('babel-plugin-transform-class-properties'));
var objectRestSpreadPlugin = _interopDefault(require('babel-plugin-transform-object-rest-spread'));
var lodashPlugin = _interopDefault(require('babel-plugin-lodash'));
var transformRuntimePlugin = _interopDefault(require('babel-plugin-transform-runtime'));
var reactIntlPlugin = _interopDefault(require('babel-plugin-react-intl'));
var removePropTypesPlugin = _interopDefault(require('babel-plugin-transform-react-remove-prop-types'));
var reactInlineElementsPlugin = _interopDefault(require('babel-plugin-transform-react-inline-elements'));
var reactConstantElements = _interopDefault(require('babel-plugin-transform-react-constant-elements'));
var es3PropertyLiterals = _interopDefault(require('babel-plugin-transform-es3-property-literals'));
var es3ExpressionLiterals = _interopDefault(require('babel-plugin-transform-es3-member-expression-literals'));
var parseJSX = _interopDefault(require('babel-plugin-syntax-jsx'));
var transformReactJSX = _interopDefault(require('babel-plugin-transform-react-jsx'));
var transformReactJSXSource = _interopDefault(require('babel-plugin-transform-react-jsx-source'));
var transformReactJSXSelf = _interopDefault(require('babel-plugin-transform-react-jsx-self'));

/* eslint-disable filenames/match-exported */
var defaults = {
  // Whether to print hints on transpilation settings which were selected.
  debug: false,

  // One of the following:
  // - "node"/nodejs"/"script"/"binary": any NodeJS related execution with wide support to last LTS aka 6.9.0
  // - "current"/"test": current NodeJS version
  // - "browser"/"web": browsers as defined by browserslist
  // - "library": ideally used for publishing libraries e.g. on NPM
  // - "es2015": same as "library" but targets es2o15 capable engines only.
  // - {}: any custom settings support by Env-Preset
  target: "nodejs",

  // Choose automatically depending on target
  modules: "auto",

  // Prefer built-ins over custom code. This mainly benefits for modern engines.
  useBuiltIns: true,

  // JSX Pragma. Default: Use React
  jsxPragma: "React.createElement",

  // Env Settings
  looseMode: true,
  specMode: false,

  // Lodash Plugin Settings
  optimizeModules: ["lodash", "async", "rambda", "recompose"],

  // Configuration for module lookup
  sourceFolder: "src",

  // Source map output
  sourceMaps: true,

  // Babel Core Settings
  comments: false,
  minified: true
};

function buildPreset() {
  var opts = 1 < arguments.length && arguments[1] !== undefined ? arguments[1] : {},
      presets = [],
      plugins = [],
      options = Object.assign({}, defaults, opts),
      envValue = process.env.BABEL_ENV || process.env.NODE_ENV || "development",
      isProduction = "production" === envValue;

  // These are the final options we use later on.


  // There is also a BROWSERSLIST_ENV

  if (options.debug) {
    console.log("- Environment:", envValue);
  }

  // Auto select test target when running in test environment and no other info is available.
  if ("test" === envValue && null == options.target) {
    options.target = "test";
  }

  var envTargets = {},
      buildDistBinary = "node" === options.target || "nodejs" === options.target || "script" === options.target || "binary" === options.target,
      buildForCurrent = "current" === options.target || "test" === options.target,
      buildForBrowserList = "browser" === options.target || "web" === options.target,
      buildAsLibrary = "library" === options.target || "es2015" === options.target,
      buildCustom = "object" == _typeof(options.target);

  if (buildDistBinary) {
    // Last stable NodeJS (LTS) - first LTS of 6.x.x was 6.9.0
    // See also: https://nodejs.org/en/blog/release/v6.9.0/
    envTargets.node = "6.9.0";
  } else if (buildForCurrent) {
    // Scripts which are directly used like tests can be transpiled for the current NodeJS version
    envTargets.node = "current";
  } else if (buildForBrowserList) {
    // Until this issue is fixed we can't use auto config detection for browserslist in babel-preset-env
    // https://github.com/babel/babel-preset-env/issues/149
    // What we do here is actually pretty clever/ytupid as we just pass over the already normalized
    // browser list to browserslist again.
    var autoBrowsers = browserslist(null, { env: isProduction ? "production" : "development" });

    // For the abstract browsers config we let browserslist find the config file
    envTargets.browsers = autoBrowsers;
  } else if (buildAsLibrary) {
    // Explicit undefined results into compilation with "latest" preset supporting a wide range of clients via ES5 output
    envTargets = undefined;
  } else if (buildCustom) {
    envTargets = options.target;
  }

  var additionalExcludes = [];

  // Exclude all es2015 features which are supported by the default es2015 babel preset.
  // This targets all es2015-capable browsers and engines.
  if ("es2015" === options.target) {
    additionalExcludes.push("transform-es2015-template-literals", "transform-es2015-literals", "transform-es2015-function-name", "transform-es2015-arrow-functions", "transform-es2015-block-scoped-functions", "transform-es2015-classes", "transform-es2015-object-super", "transform-es2015-shorthand-properties", "transform-es2015-duplicate-keys", "transform-es2015-computed-properties", "transform-es2015-for-of", "transform-es2015-sticky-regex", "transform-es2015-unicode-regex", "check-es2015-constants", "transform-es2015-spread", "transform-es2015-parameters", "transform-es2015-destructuring", "transform-es2015-block-scoping", "transform-es2015-typeof-symbol", "transform-es2015-modules-commonjs", "transform-es2015-modules-systemjs", "transform-es2015-modules-amd", "transform-es2015-modules-umd"

    // This is already excluded by default
    // "transform-regenerator"
    );
  }

  if (options.debug) {
    if ("es2015" === options.target) {
      console.log("- Environment Targets: ES2015 capable");
    } else {
      console.log("- Environment Targets:", envTargets);
    }
  }

  if ("auto" === options.modules || null == options.modules) {
    if ("node" === options.target || "nodejs" === options.target || "script" === options.target || "binary" === options.target || "test" === options.target) {
      options.modules = "commonjs";
    } else if ("library" === options.target || "browser" === options.target) {
      // Libraries should be published as EcmaScript modules for tree shaking support
      // For browser targets we typically use tools like Webpack which benefit from EcmaScript modules, too.
      options.modules = false;
    } else {
      // Best overall support when nothing other is applicable
      options.modules = "commonjs";
    }
  }

  if (options.debug) {
    console.log("- Module Settings:", false === options.modules ? "ESM" : options.modules);
    console.log("- Transpilation Compliance:", options.specMode ? "SPEC" : options.looseMode ? "LOOSE" : "DEFAULT");
  }

  presets.push([envPreset, {
    // Setting this to false will not transform modules.
    modules: options.modules,

    // Prefer built-ins which also prefers global polyfills which is the right thing to do
    // for most scenarios like SPAs and NodeJS environments.
    useBuiltIns: options.useBuiltIns,

    // Options to tweak the details of the implementation. If both are `false` the environment
    // preset is executed in default mode.
    loose: options.looseMode,
    spec: options.specMode,

    // Debug output of features, plugins and presets which are enabled.
    // debug: true,

    // We prefer the transpilation of the "fast-async" plugin over the
    // slower and more complex Babel internal implementation.
    exclude: ["transform-regenerator", "transform-async-to-generator"].concat(additionalExcludes),

    // Differ between development and production for our scope.
    // NodeJS is generally fine in development to match the runtime version which is currently installed.
    targets: envTargets
  }]);

  // Support for Flowtype Parsing
  presets.push(flowPreset);

  // Support for new @import() syntax
  plugins.push(dynamicImportPlugin);

  // Transpile the parsed import() syntax for compatibility in older environments.
  if (buildForCurrent || buildDistBinary) {
    // Compiles import() to a deferred require() for NodeJS
    plugins.push(dynamicImportNode);
  } else if (buildAsLibrary) {
    // This is our alternative appeoach for now which "protects" these imports from Rollup
    // for usage in Webpack later on. In detail it transpiles `import()` to `require.ensure()` before
    // it reaches RollupJS's bundling engine.
    // https://github.com/airbnb/babel-plugin-dynamic-import-webpack
    plugins.push(dynamicImportWebpack);
  }

  // Improve some ES3 edge case to make code parseable by older clients
  // e.g. when using reserved words as keys like "catch"
  plugins.push(es3ExpressionLiterals, es3PropertyLiterals);

  // Optimization for cheery-picking from lodash, asyncjs, ramba and recompose.
  // Auto cherry-picking es2015 imports from path imports.
  // https://www.npmjs.com/package/babel-plugin-lodash
  // https://github.com/acdlite/recompose#using-babel-lodash-plugin
  plugins.push([lodashPlugin, { id: options.optimizeModules }]);

  // Supports loading files in source folder without relative folders
  // https://github.com/tleunen/babel-plugin-module-resolver
  plugins.push([moduleResolver, {
    alias: {
      "~": path.resolve(appRootDir.get(), options.sourceFolder)
    }
  }]);

  // Alternative to Babel Regenerator
  // Implements the ES7 keywords async and await using syntax transformation at compile-time, rather than generators.
  // https://www.npmjs.com/package/fast-async
  plugins.push([fastAsyncPlugin, {
    useRuntimeModule: true
  }]);

  // Support for ES7 Class Properties (currently stage-2)
  // class { handleClick = () => { } }
  plugins.push(classPropertiesPlugin);

  // Support for Object Rest Spread `...` operator in objects.
  // { ...todo, completed: true }
  plugins.push([objectRestSpreadPlugin, {
    useBuiltIns: options.useBuiltIns
  }]);

  // Allow Babel to parse JSX
  plugins.push(parseJSX);

  // Improve JSX debug capabilities
  if (!isProduction) {
    // Adds component stack to warning messages
    plugins.push(transformReactJSXSource);

    // Adds __self attribute to JSX which React will use for some warnings
    plugins.push(transformReactJSXSelf);
  }

  // Transform JSX and prefer built-in methods
  plugins.push([transformReactJSX, {
    useBuiltIns: options.useBuiltIns,
    pragma: options.jsxPragma
  }]);

  // Use helpers, but not polyfills, in a way that omits duplication.
  // For polyfills better use polyfill.io or another more sophisticated solution.
  plugins.push([transformRuntimePlugin, {
    helpers: true,
    regenerator: false,
    polyfill: false,
    useBuiltIns: options.useBuiltIns,
    useESModules: true
  }]);

  if (isProduction) {
    // Cleanup descriptions for translations from compilation output
    plugins.push(reactIntlPlugin);

    // Remove prop types from our code
    plugins.push([removePropTypesPlugin, {
      removeImport: true
    }]);

    // Replaces the React.createElement function with one that is
    // more optimized for production.
    // NOTE: Symbol needs to be polyfilled.
    plugins.push(reactInlineElementsPlugin);

    // Hoists element creation to the top level for subtrees that
    // are fully static, which reduces call to React.createElement
    // and the resulting allocations. More importantly, it tells
    // React that the subtree hasn’t changed so React can completely
    // skip it when reconciling.
    plugins.push(reactConstantElements);
  }

  // Assemble final config
  return {
    // Babel basic configuration
    comments: options.comments,
    compact: true,
    minified: options.minified,

    // Whether to enable source maps
    sourceMaps: options.sourceMaps,

    // And all the previously built lists of presets and plugins
    presets: presets,
    plugins: plugins
  };
}

module.exports = buildPreset;
//# sourceMappingURL=node.commonjs.js.map
